import usb

from rgb_control.rgb_utils import RGBProfile


class UsbDevice:
    bm_request_type: int = 0x00000021
    b_request: int = 0x09
    w_value: int = 0x00000307
    w_index: int = 2

    id_vendor: int
    id_product: int

    def __init__(self, id_vendor: int, id_product: int):
        self.id_vendor = id_vendor
        self.id_product = id_product
        self.__device = None

        self.__init_device()

    def __init_device(self) -> None:
        self.__device = usb.core.find(
            idVendor=self.id_vendor, idProduct=self.id_product
        )
        if self.__device is None:
            raise ValueError("Keyboard not found!")

    def detach(self) -> None:
        for config in self.__device:
            for i in range(config.bNumInterfaces):
                if self.__device.is_kernel_driver_active(i):
                    self.__device.detach_kernel_driver(i)
        self.__device.set_configuration()

    def release(self) -> None:
        for config in self.__device:
            for i in range(config.bNumInterfaces):
                usb.util.release_interface(self.__device, i)
                self.__device.attach_kernel_driver(i)

    def set_value(self, hex_value: str) -> None:
        try:
            self.__device.ctrl_transfer(
                self.bm_request_type,
                self.b_request,
                self.w_value,
                self.w_index,
                bytes.fromhex(hex_value),
            )
        except ValueError:
            self.release()
            raise


class Keyboard:
    def __init__(self, id_vendor: int, id_product: int):
        self.__keyboard_device = UsbDevice(id_vendor, id_product)

    def set_rgb_highlight(self, rgb_profile: RGBProfile) -> None:
        self.__keyboard_device.detach()
        for package_data in rgb_profile.packages_data:
            try:
                self.__keyboard_device.set_value(package_data)
            except usb.core.USBError:
                self.__keyboard_device.release()
                raise
        self.__keyboard_device.release()



import argparse
import functools

from rgb_control.device import Keyboard
from rgb_control.rgb_utils import HexColor, RGBProfile

ID_VENDOR: int = 0x09DA
ID_PRODUCT: int = 0xFA10

all_int = functools.partial(int, base=0)


def init_argparser():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-c",
        "--color",
        type=str,
        help="color in hex format. example: 00ff00",
        required=True,
    )
    parser.add_argument(
        "-id_v",
        "--id_vendor",
        type=all_int,
        help="vendor id in hex format, default: 0x09DA".format(ID_VENDOR),
        default=ID_VENDOR,
    )
    parser.add_argument(
        "-id_p",
        "--id_product",
        type=all_int,
        help="product id in hex format, default: 0xFA10".format(ID_PRODUCT),
        default=ID_PRODUCT,
    )
    return parser


def set_rgb_highlight(color, id_vendor, id_product):
    rgb_keyboard = Keyboard(id_vendor, id_product)
    rgb_color = HexColor(color)
    rgb_profile = RGBProfile(rgb_color)
    rgb_keyboard.set_rgb_highlight(rgb_profile)


def main():
    parser = init_argparser()
    args = parser.parse_args()
    set_rgb_highlight(**vars(args))


if __name__ == "__main__":
    main()



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="rgb-control",
    version="0.0.1",
    author="Alexandr Kuzhel",
    author_email="alexkuzhel.biz@gmail.com",
    description="The script allows you to set the RGB backlight color on A4Tech Bloody keyboards.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    entry_points={
        "console_scripts": ["rgb-control=rgb_control.main:main"],
    },
    url="https://github.com/apo5tol/bloody_keyboard_rgb_control",
    packages=setuptools.find_packages(),
    install_requires=[
        "pyusb==1.1.0",
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX :: Linux",
    ],
    python_requires=">=3.6",
)